import re


def run(results, raw_paste_data, paste_object):
    # Use the rule name to determine what postprocess to do

    # Get total unique emails.

    all_emails = re.findall(r'\b([\w-]+(?:\.[\w-]+)*@[\w-]+(?:\.[\w-]+)*\.[a-zA-Z-]+[\w-])\b', raw_paste_data)
    domain_list = []
    for email_address in all_emails:
        email_domain = email_address.split("@")
        domain_list.append(email_domain[-1])

    unique_emails = set(all_emails)
    unique_domains = set(domain_list)
    # We can filter some of the false positives from the yara match here

    if len(unique_emails) < 10:
        paste_object["results"] = []

    # Get unique domain count
    # Update the json
    paste_object["total_emails"] = len(all_emails)
    paste_object["unique_emails"] = len(unique_emails)
    paste_object["unique_domains"] = len(unique_domains)

    # Send the updated json back
    return paste_object



import logging
import re
from datetime import datetime
from typing import List, Any, Dict, Union, Pattern

from pastehunter.common import base62_decode, base62_encode
from pastehunter.inputs.base_input import BasePasteSite

logger = logging.getLogger('pastehunter')


class IxDotIoSite(BasePasteSite):
    # Yeah, yeah, I know, no regex for HTML parsing...
    # If we end up doing a lot more of this, then maybe we'll use beautifulsoup or something.
    # Capturing groups:
    # 1. Paste ID
    # 2. Timestamp
    _ITEM_ID_RE: Pattern = re.compile('<div class="t">[\\sa-zA-Z0-9]+'
                                         '<a href="/(.*?)">\\[r][^\r\n]+'
                                         '\\s+@ (.*?)[\r\n]')

    def __init__(self, conf):
        self.conf = conf
        self.site = "ix.io"
        url_main = "http://" + self.site
        self.url_recent = url_main + "/user/"
        self.view_pattern = url_main + "/{}/"
        self.raw_pattern = url_main + "/{}"
        self.url = None

    def remap_raw_item(self, raw_item: [str, Dict]) -> Dict[str, Any]:
        pid = raw_item['pid']
        paste_data = {
            # at a
            'filename': str(pid),
            'confname': 'ixio',
            'pastesite': self.site,
            'pasteid': pid,
        }
        # Timezone is UTC/Zulu
        date = datetime.strptime(raw_item['date'], '%a %b %d %H:%M:%S %Y').isoformat()
        paste_data['@timestamp'] = date
        encoded_pid = self.get_paste_id(paste_data)
        paste_data['scrape_url'] = self.raw_pattern.format(encoded_pid)
        return paste_data

    def get_paste_for_id(self, paste_id: Any) -> str:
        self.make_request(self.raw_pattern.format(paste_id))

    def get_paste_id(self, paste_obj: Dict[str, Any]) -> str:
        decoded = paste_obj.get('pasteid')
        return base62_encode(decoded)

    def get_recent_items(self, input_history: List[str]):

        history = []
        paste_list = []
        try:
            recent_page = self.make_request(self.url_recent)
            item_data = self.get_data_for_page(recent_page.text)

            for val in item_data:
                # Track paste ids to prevent dupes
                pid = val['pid']
                history.append(pid)
                if pid in input_history:
                    continue
                paste_data = self.remap_raw_item(val)
                paste_list.append(paste_data)

            return paste_list, history

        except Exception as e:
            logger.error("Unable to parse ixio items: {0}".format(e))
            return paste_list, history

    def get_data_for_page(self, page_data: str) -> List[Dict[str, Union[int, str]]]:
        page: List[Dict[str, Union[int, str]]] = []
        last_item_id = -1
        regex_matches = self._ITEM_ID_RE.findall(page_data)
        # We are going to reverse the order because ix pages are structured newest -> oldest, and this makes it simpler.
        regex_matches.reverse()
        for encoded_id, created_at in regex_matches:
            # Okay so the logic here is a bit tricky. Basically, ix's all user page only returns anonymous pastes
            # BUT! We can infer the paste ids that aren't present by filling in the blanks, because ix IDs are
            # incremental. So first, we base62 decode the value so we can use it as an int
            item_id = base62_decode(encoded_id)
            # Then, we check if we've seen another value. If this is our first, we can skip a lot of this logic.
            # (we probably don't want to go back and grab every ix paste historically for most use cases)
            if last_item_id == -1:
                page.append({'pid': item_id, 'date': created_at})
                last_item_id = item_id
            # If there has been a delta, let's traverse it.
            elif item_id - last_item_id > 1:
                # We've already hit last_item_id so we skip that and fill in the delta
                for i in range(last_item_id + 1, item_id + 1):
                    # Copy the created date as a best guess
                    page.append({'pid': i, 'date': created_at})
                last_item_id = item_id
            else:
                # If there's no delta, just add this nromally
                page.append({'pid': item_id, 'date': created_at})
                last_item_id = item_id
        return page


def recent_pastes(conf, input_history):
    site = IxDotIoSite(conf)

    # populate vars from config
    return site.get_recent_items(input_history)



import logging
import requests
from pastehunter.common import parse_config

logger = logging.getLogger('pastehunter')

config = parse_config()


class SlackOutput():
    def __init__(self):
        self.valid = True
        self.webhook_url = config['outputs']['slack_output']['webhook_url']
        self.accepted_rules = config['outputs']['slack_output']['rule_list']

        if self.webhook_url == '':
            logging.error("Slack Webhook not configured")
            self.valid = False
        if self.webhook_url == '':
            logging.error("No Rules configured to alert")

    def store_paste(self, paste_data):
        if self.valid:
            send = ('all' in self.accepted_rules)

            for rule in self.accepted_rules:
                if rule in paste_data['YaraRule']:
                    send = True

            if send:
                json_data = {
                    "text": "Pastehunter alert!",
                    "attachments": [
                        {
                            "fallback": "Plan a vacation",
                            "author_name": "PasteHunter",
                            "title": "Paste ID {0}".format(paste_data['pasteid']),
                            "text": "Yara Rule {0} Found on {1}\n\r{2}".format(paste_data['YaraRule'], paste_data['pastesite'], paste_data['scrape_url'])
                        }
                    ]
                }

                req = requests.post(self.webhook_url, json=json_data)
                if req.status_code == 200 and req.text == 'ok':
                    logger.debug("Paste sent to slack")
                else:
                    logger.error(
                        "Failed to post to slack Status Code {0}".format(req.status_code))



import logging
import os
import datetime
from pastehunter.common import parse_config

logger = logging.getLogger('pastehunter')
config = parse_config()


class CSVOutput(object):
    def __init__(self):
        base_path = config['outputs']['csv_output']['output_path']
        # Get todays CSV
        dtg = datetime.date.today().strftime("%Y-%m-%d")
        csv_name = 'pastes_{0}.csv'.format(dtg)
        self.csv_path = os.path.join(base_path, csv_name)

        if not os.path.exists(base_path):
            try:
                os.makedirs(base_path)
                self.test = True
            except OSError as e:
                logger.error("Unable to create CSV Path: {}".format(e))
                self.test = False
        else:
            self.test = True

    def store_paste(self, paste_data):
        if self.test:
            # date, _id, YaraRule, raw_url
            csv_line = '{0},{1},{2},{3},{4}'.format(paste_data['@timestamp'],
                                                paste_data['pasteid'],
                                                paste_data['YaraRule'],
                                                paste_data['scrape_url'],
                                                paste_data['pastesite'])
            with open(self.csv_path, 'a') as out:
                out.write('{0}\n'.format(csv_line))
        else:
            logging.error("CSV Output Error. Output path '{}' was never created.".format(self.csv_path))



import logging
import re
from datetime import datetime
from time import sleep
from typing import Any, Dict, Optional, List, Union

from pastehunter.inputs.base_input import BasePasteSite

logger = logging.getLogger('pastehunter')


class SlexyPasteSite(BasePasteSite):

    def __init__(self, conf):
        self.url = None
        self.site = "slexy.org"
        url_slexy = "https://" + self.site
        self.url_recent = url_slexy + "/recent"
        self.url_view = url_slexy + "/view"
        self.url_raw = url_slexy + "/raw"
        self.fetch_timeout = conf.get('fetch_timeout', 15)

    def make_request(self, url: str, timeout: Optional[int] = 15, headers: Optional[Dict[str, Any]] = None):
        req = super(SlexyPasteSite, self).make_request(url, timeout, {
            'Referer': self.url_recent,
            'User-Agent': 'PasteHunter'
        })

        ratelimit_limit = int(req.headers.get('RateLimit-Limit', 30))
        remaining = int(req.headers.get('RateLimit-Remaining', 30))
        logger.debug('Remaining Slexy Ratelimit: {0}'.format(remaining))

        if req.status_code == 429:
            delay = req.headers.get('Retry-After', 60)
            sleep(delay)
            return self.make_request(url, timeout)
        # If ratelimit_limit = 60, 60/60 = 1
        # If ratelimit_limit = 30, 60/30 = 2
        sleep(30 / ratelimit_limit)
        return req.text

    def get_timestamp(self, data):
        pattern = 'Timestamp: <b>(.*?)</b>'
        ts = re.findall(pattern, data)[0]
        return datetime.strptime(ts, "%Y-%m-%d %H:%M:%S %z").isoformat()

    def get_paste_id(self, paste_obj: Dict[str, Any]) -> Union[str, int]:
        return paste_obj.get('pasteid')

    def remap_raw_item(self, raw_item: [str, Dict]) -> Dict[str, Any]:
        timestamp = self.get_timestamp(raw_item)
        paste_id = self.get_paste_id(raw_item)
        raw_url = self.get_raw_link(raw_item, paste_id)
        self.get_paste_id(raw_item)
        return {
            'confname': 'slexy',
            'scrape_url': raw_url,
            'pasteid': paste_id,
            'pastesite': self.site,
            '@timestamp': timestamp
        }

    def get_raw_data(self, raw_url):
        return self.make_request(raw_url, self.fetch_timeout)

    def get_paste_for_id(self, paste_id: Any) -> str:
        return self.make_request("%s/%s" % (self.url_view, paste_id), self.fetch_timeout)

    def get_raw_link(self, data, pid):
        pattern = '<a href="/raw/%s(.*?)"' % pid
        token = re.findall(pattern, data)[0]
        return "%s/%s%s" % (self.url_raw, pid, token)

    def get_recent_items(self, input_history: List[str]):
        data = self.make_request(self.url_recent, self.fetch_timeout)
        pids = re.findall('<td><a href="/view/(.*?)">', data)
        return list(set(pids))


def recent_pastes(conf, input_history):
    history = []
    paste_list = []
    my_scraper = SlexyPasteSite(conf['inputs']['slexy'])
    recent_pids = my_scraper.get_recent_items(input_history)
    pid_to_process = set()
    for pid in recent_pids:
        if pid in input_history:
            history.append(pid)
        else:
            pid_to_process.add(pid)
    try:
        for pid in pid_to_process:
            paste_data = my_scraper.get_paste_for_id(pid)
            raw = my_scraper.get_raw_link(paste_data, pid)
            paste_data = {
                'confname': 'slexy',
                'scrape_url': raw,
                'pasteid': pid,
                'pastesite': my_scraper.site,
                '@timestamp': my_scraper.get_timestamp(paste_data)
            }
            paste_list.append(paste_data)
        return paste_list, history
    except Exception as e:
        logger.error("Unable to parse paste results: %s", e)
        return paste_list, history



from pastehunter.common import base62_decode, base62_encode


def test_b62_encode():
    assert base62_encode(622708) == '2BZG'
    assert base62_encode(622707) == '2BZF'


def test_b62_decode():
    assert base62_decode('1') == 1
    assert base62_decode('a') == 10
    assert base62_decode('2BZF') == 622707
    assert base62_decode('2BZG') == 622708


import requests
import math
import logging
from datetime import datetime

# Set some logging options
logger = logging.getLogger('pastehunter')
logging.getLogger('requests').setLevel(logging.ERROR)

api_uri = 'https://api.github.com/gists/public'
api_version = 'application/vnd.github.v3+json'  # Set Accept header to force api v3

# Some people use gists to store large blobs of data every 17 minutes. This just slows down the kibana UI



def recent_pastes(conf, input_history):
    oauth_token = conf['inputs']['gists']['api_token']
    gist_limit = conf['inputs']['gists']['api_limit']
    headers = {'user-agent': 'PasteHunter',
               'Accept': api_version,
               'Authorization': 'token {0}'.format(oauth_token)}

    # calculate number of pages
    page_count = int(math.ceil(gist_limit / 100))

    result_pages = []
    history = []
    paste_list = []

    gist_file_blacklist = conf['inputs']['gists']['file_blacklist']
    gist_user_blacklist = conf['inputs']['gists']['user_blacklist']

    try:
        # Get the required amount of entries via pagination
        for page_num in range(1, page_count + 1):
            url = '{0}?page={1}&per_page=100'.format(api_uri, page_num)
            logger.debug("Fetching page: {0}".format(page_num))
            req = requests.get(url, headers=headers)
            # Check some headers
            reset_date = datetime.utcfromtimestamp(float(req.headers['X-RateLimit-Reset'])).isoformat()
            # logging.info("Limit Reset: {0}".format(reset_date))
            logger.info("Remaining Limit: {0}. Resets at {1}".format(req.headers['X-RateLimit-Remaining'],
                                                                      reset_date))

            if req.status_code == 200:
                result_pages.append(req.json())

            if req.status_code == 401:
                logger.error("Auth Failed")

            elif req.status_code == 403:
                logger.error("Login Attempts Exceeded")

        # Parse results

        for page in result_pages:
            for gist_meta in page:
                # Track paste ids to prevent dupes
                history.append(gist_meta['id'])
                if gist_meta['id'] in input_history:
                    continue

                if gist_meta['user'] in gist_user_blacklist:
                    logger.info("Blacklisting Gist from user: {0}".format(gist_meta['owner']['login']))
                    continue

                for file_name, file_meta in gist_meta['files'].items():

                    if file_name in gist_file_blacklist:
                        logger.info("Blacklisting Paste {0}".format(file_name))
                        continue

                    gist_data = file_meta
                    gist_data['confname'] = 'gists'
                    gist_data['@timestamp'] = gist_meta['created_at']
                    gist_data['pasteid'] = gist_meta['id']
                    gist_data['user'] = gist_meta['user']
                    gist_data['pastesite'] = 'gist.github.com'
                    gist_data['scrape_url'] = file_meta['raw_url']
                    # remove some origional keys just to keep it a bit cleaner
                    del gist_data['raw_url']
                    paste_list.append(gist_data)

        # Return results and history
        return paste_list, history
    except Exception as e:
        logger.error("Unable to parse paste results: {0}".format(e))
        return paste_list, history



import io
import logging
import requests
from pastehunter.common import parse_config
conf = parse_config()

logger = logging.getLogger('pastehunter')

def upload_file(raw_file, paste_object):
    viper_ip = conf["sandboxes"]["viper"]["api_host"]
    viper_port = conf["sandboxes"]["viper"]["api_port"]
    viper_host = 'http://{0}:{1}'.format(viper_ip, viper_port)

    submit_file_url = '{0}/tasks/create/file'.format(viper_host)
    files = {'file': ('{0}.exe'.format(paste_object["pasteid"]), io.BytesIO(raw_file))}
    submit_file = requests.post(submit_file_url, files=files).json()

    # Send any updated json back
    return paste_object



import requests
import math
import logging
from datetime import datetime

# Set some logging options
logger = logging.getLogger('pastehunter')
logging.getLogger('requests').setLevel(logging.ERROR)

# Test API Key from the docs - U4DMV*8nvpm3EOpvf69Rxw((
# https://api.stackexchange.com/2.2/questions?key=U4DMV*8nvpm3EOpvf69Rxw((&site=stackoverflow&page=1&pagesize=100&order=desc&sort=creation&filter=default



def recent_pastes(conf, input_history):
    api_key = conf['inputs']['stackexchange']['api_key']
    api_scrape = conf['inputs']['stackexchange']['api_scrape']
    site_list = conf['inputs']['stackexchange']['site_list']
    store_filter = conf['inputs']['stackexchange']['store_filter']
    question_body_filter = '!bA1dOlliDM)pi9'
    pagesize = 100 # Default = 30
    headers = {'user-agent': 'PasteHunter'}

    if api_key == '':
        logger.error("No API Key configured for StackExchange Access")
        return [], []

    result_pages = []
    history = []
    paste_list = []

    try:
        
        # For each of the stack sites we want to query
        for site in site_list:
            logger.info("Query Stack Exchange site: {0}".format(site))

            # Create the API uri
            scrape_uri = '{0}?key={1}&site={2}&page=1&pagesize=100&order=desc&sort=creation&filter={3}'.format(api_scrape, api_key, site, store_filter)
            # Get some pastes and convert to json
            # Get last 'paste_limit' pastes
            paste_list_request = requests.get(scrape_uri)
    
            # ToDo: Add an API rate test in here. 
            paste_list_json = paste_list_request.json()
            
            if "error_id" in paste_list_json:
                logging.error("StackExchange API Error: {0}".format(paste_list_json['error_message']))
                return [], []
            
            
    
            for question in paste_list_json['items']:
                # Track question ids to prevent dupes
                history.append(question['question_id'])
                if question['question_id'] in input_history:
                    continue
    
                # Create a new question dict for us to normalize
                question_data = question
                question_data['filename'] = ''
                question_data['confname'] = "stackexchange"
                # Force type to string else it breaks ES Index mappings
                question_data['pasteid'] = str(question['question_id']) 
                question_data['pastesite'] = site
                # Set the raw uri to avoid breaking other things. Defaults to empty if not found
                question_data['scrape_url'] = question.get('link', '')
                # Get the author and then trim the data we store. 
                question_data['username'] = question['owner']['display_name']
                del question_data['owner']
                # Add a date field that kibana will map
                date = datetime.utcfromtimestamp(float(question_data['creation_date'])).isoformat()
                question_data['@timestamp'] = date
                paste_list.append(question_data)
            
            
            # Record API Quota on last call to save some logging. 
            quota_max = paste_list_json['quota_max']
            quota_remaining = paste_list_json['quota_remaining']
        
        logger.info("Used {0} of {1} of StackExchange api quota".format(quota_remaining, quota_max))
        # Return the pastes and update history
        return paste_list, history

    except Exception as e:
        logger.error("Unable to parse question results: {0}".format(e))
        return paste_list, history


import smtplib
import email.encoders
import email.header
import email.mime.base
import email.mime.multipart
import email.mime.text
from email.utils import formatdate
from email.mime.multipart import MIMEMultipart
import json
import logging

from pastehunter.common import parse_config
logger = logging.getLogger('pastehunter')

config = parse_config()

class SMTPOutput():
    def __init__(self):
        smtp_object = config['outputs']['smtp_output']
        self.smtp_host = smtp_object['smtp_host']
        self.smtp_port = smtp_object['smtp_port']
        self.smtp_security = smtp_object['smtp_security']
        self.smtp_user = smtp_object['smtp_user']
        self.smtp_pass = smtp_object['smtp_pass']
        if 'recipients' in smtp_object:
            self.recipients = smtp_object['recipients']
        else:
            # maintain compatibility with older single recipient config format
            self.recipients = {'main': {'address': smtp_object['recipient'],
                                        'rule_list': smtp_object['rule_list'],
                                        'mandatory_rule_list': []}}


    def _send_mail(self, send_to_address, paste_data):
        logger.info("crafting email for {0}".format(send_to_address))

        # Create the message
        msg = MIMEMultipart()
        msg['Subject'] = 'PasteHunter Alert {0}'.format(', '.join(paste_data['YaraRule']))
        msg['From'] = self.smtp_user
        msg['To'] = send_to_address
        msg["Date"] = formatdate(localtime=True)

        # Attach the body
        body = 'Rules : {0}\n' \
               'Paste : {1} from {2}\n\n' \
               'A Copy of the paste has been attached'.format(', '.join(paste_data['YaraRule']),
                                                              paste_data['pasteid'],
                                                              paste_data['pastesite'])
        msg.attach(email.mime.text.MIMEText(body, 'plain'))

        # Attach the raw paste as JSON
        attachment = email.mime.base.MIMEBase('application', 'json')
        json_body = json.dumps(paste_data)
        attachment.set_payload(json_body)
        email.encoders.encode_base64(attachment)
        attachment.add_header('Content-Disposition', 'attachment; filename="Alert-{0}.json"'.format(paste_data['pasteid']))
        msg.attach(attachment)

        # Connect to the SMTP server and send
        if self.smtp_security == 'ssl':
            smtp_conn = smtplib.SMTP_SSL(self.smtp_host, self.smtp_port)
        else:
            smtp_conn = smtplib.SMTP(self.smtp_host, self.smtp_port)
        smtp_conn.ehlo()
        if self.smtp_security == 'tls':
            smtp_conn.starttls()
        smtp_conn.login(self.smtp_user, self.smtp_pass)
        smtp_conn.send_message(msg)
        smtp_conn.quit()

        logger.info("Sent mail to {0} with rules {1}".format(send_to_address,
                                                              ', '.join(paste_data['YaraRule'])))


    def _check_recipient_rules(self, paste_data, recipient_name):

            # Read each recipient's config
            recipient = self.recipients[recipient_name]
            recipient_address = recipient['address']
            all_rules_mandatory = False
            if len(recipient['mandatory_rule_list']):
                recipient_rule_list = recipient['mandatory_rule_list']
                all_rules_mandatory = True
            else:
                recipient_rule_list = recipient['rule_list']

            # Check if the recipient has special rule 'all' meaning it gets all alerts
            if 'all' in recipient_rule_list:
                self._send_mail(recipient_address, paste_data)
                return

            # Check if all of the recipient's rules need to be found in the alert
            if all_rules_mandatory:
                if all(elem in paste_data['YaraRule'] for elem in recipient_rule_list):
                    self._send_mail(recipient_address, paste_data)
                return

            # Nominal case, check if at least one rule is found in the alert
            if any(elem in paste_data['YaraRule'] for elem in recipient_rule_list):
                self._send_mail(recipient_address, paste_data)
                return


    def store_paste(self, paste_data):
        for recipient_name in self.recipients:
            self._check_recipient_rules(paste_data, recipient_name)



import logging
from twilio.rest import Client
from pastehunter.common import parse_config

logger = logging.getLogger('pastehunter')
config = parse_config()

class TwilioOutput(object):
    def __init__(self):
        self.account_sid = config['outputs']['twilio_output']['account_sid']
        self.auth_token = config['outputs']['twilio_output']['auth_token']
        self.twilio_sender = config['outputs']['twilio_output']['twilio_sender']
        self.recipient_list = config['outputs']['twilio_output']['recipient_list']
        self.accepted_rules = config['outputs']['twilio_output']['rule_list']
        self.message_type = 'sms' # Whatsapp is still in beta on twilio.
        try:
            self.client = Client(self.account_sid, self.auth_token)
            self.test = True
        except Exception as e:
            logging.error("Unable to create twilio Client: {0}".format(e))
            self.test = False


    def store_paste(self, paste_data):
        if self.test:


            send = ('all' in self.accepted_rules)

            for rule in self.accepted_rules:
                if rule in paste_data['YaraRule']:
                    send = True

            if send:
                message_body = "Yara Rule {0} Found on {1}\n\r{2}".format(
                    paste_data['YaraRule'], 
                    paste_data['pastesite'], 
                    paste_data['scrape_url']
                    )

                logger.debug("Sending Twilio Message")
                if self.message_type == 'sms':
                    for recipient in self.recipient_list:
                        try:
                            message = self.client.messages.create( 
                                                        from_=self.twilio_sender,  
                                                        body=message_body,      
                                                        to=recipient 
                                                    )
                            logging.debug("Sent twilio message with ID: {0}".format(message.sid))
                        except Exception as e:
                            logging.error(e)

                elif self.message_type == 'whatsapp':
                    for recipient in self.recipient_list:
                        try:
                            message = self.client.messages.create( 
                                                        from_='whatsapp:{0}'.format(self.twilio_sender),  
                                                        body=message_body,      
                                                        to='whatsapp:{0}'.format(recipient) 
                                                    )
                            logging.debug("Sent twilio message with ID: {0}".format(message.sid))
                        except Exception as e:
                            logging.error(e)
                else:
                    logging.error("No Valid twilio message type found")



import requests
import logging

logger = logging.getLogger('pastehunter')

def recent_pastes(conf, input_history):
    # populate vars from config
    paste_limit = conf['inputs']['dumpz']['paste_limit']
    api_scrape = conf['inputs']['dumpz']['api_scrape']
    history = []
    paste_list = []
    try:
        # Create the API uri
        scrape_uri = '{0}?limit={1}'.format(api_scrape, paste_limit)
        # Get some pastes and convert to json
        # Get last 'paste_limit' pastes
        paste_list_request = requests.get(scrape_uri)
        paste_list_json = paste_list_request.json()

        for paste in paste_list_json['dumps']:
            # Track paste ids to prevent dupes
            history.append(paste['id'])
            if paste['id'] in input_history:
                continue

            # We don't want password protected pastes
            if paste['pwd'] == 1:
                continue

            # Create a new paste dict for us to normalize
            paste_data = paste
            paste_data['confname'] = 'dumpz'
            paste_data['pasteid'] = paste['id']
            paste_data['pastesite'] = 'dumpz.org'

            #paste_data['scrape_url'] = '{0}{1}'.format(conf['dumpz']['api_raw'], paste['id'])

            paste_data['scrape_url'] = 'https://dumpz.org/{0}/text/'.format(paste['id'])

            # Add a date field that kibana will map
            paste_data['@timestamp'] = paste_data['date']
            paste_list.append(paste_data)
        return paste_list, history

    except Exception as e:
        logger.error("Unable to parse paste results: {0}".format(e))
        return paste_list, history


from typing import Any, Dict, Union, Optional

import requests
import logging
from datetime import datetime

from pastehunter.inputs.base_input import BasePasteSite

logger = logging.getLogger('pastehunter')


class PastebinPasteSite(BasePasteSite):

    def __init__(self, conf):
        self.conf = conf

    def remap_raw_item(self, raw_item: Dict) -> Dict[str, Any]:
        # Create a new paste dict for us to normalize
        pid = self.get_paste_id(raw_item)
        paste_data = raw_item
        paste_data['filename'] = pid
        paste_data['confname'] = 'pastebin'
        paste_data['pasteid'] = pid
        paste_data['pastesite'] = 'pastebin.com'
        # Add a date field that kibana will map
        date = datetime.utcfromtimestamp(float(paste_data['date'])).isoformat()
        paste_data['@timestamp'] = date
        return paste_data

    def make_request(self, url: str, timeout: Optional[int] = 10, headers: Optional[Dict[str, Any]] = None):
        paste_list_request = super(PastebinPasteSite, self).make_request(url, timeout, headers)

        # Check to see if our IP is whitelisted or not.
        if 'DOES NOT HAVE ACCESS' in paste_list_request.text:
            logger.error("Your IP is not whitelisted visits 'https://pastebin.com/doc_scraping_api'")
            return None
        return paste_list_request

    def get_paste_for_id(self, paste_id: Any) -> str:
        pass

    def get_paste_id(self, paste_obj: Dict[str, Any]) -> Union[str, int]:
        return paste_obj['key']

    def get_recent_items(self, input_history):
        paste_limit = self.conf['inputs']['pastebin']['paste_limit']
        api_scrape = self.conf['inputs']['pastebin']['api_scrape']

        history = []
        paste_list = []
        try:
            # Create the API uri
            scrape_uri = '{0}?limit={1}'.format(api_scrape, paste_limit)
            # Get some pastes and convert to json
            # Get last 'paste_limit' pastes

            paste_list_request = self.make_request(scrape_uri)

            # IP not whitelisted
            if not paste_list_request:
                return [], []

            paste_list_json = paste_list_request.json()

            for paste in paste_list_json:
                pid = self.get_paste_id(paste)
                # Track paste ids to prevent dupes
                history.append(pid)
                if pid in input_history:
                    continue

                paste_data = self.remap_raw_item(paste)
                paste_list.append(paste_data)

            return paste_list, history

        except Exception as e:
            logger.error("Unable to parse paste results: {0}".format(e))
            return paste_list, history


def recent_pastes(conf, input_history):
    site = PastebinPasteSite(conf)
    # populate vars from config
    return site.get_recent_items(input_history)






import json
import logging
import os

from pastehunter.common import parse_config

logger = logging.getLogger('pastehunter')

config = parse_config()

class JsonOutput():
    def __init__(self):
        base_path = config['outputs']['json_output']['output_path']
        self.json_path = base_path
        if not os.path.exists(base_path):
            try:
                os.makedirs(base_path)
                self.test = True
            except OSError as e:
                logger.error("Unable to create Json Path: {0}".format(e))
                self.test = False
        else:
            self.test = True

    def store_paste(self, paste_data):
        if not config['outputs']['json_output']['store_raw']:
            del paste_data['raw_paste']

        if self.test:
            json_file = os.path.join(self.json_path, str(paste_data['pasteid']))
            with open(json_file, 'w') as out:
                out.write(json.dumps(paste_data, indent=4))
        else:
            logger.error("JsonOutput Error")



from pastehunter.postprocess.post_email import run as run_email


def test_email_filter_post():
    email_dummy = """
    garbage text sdgusdfjhuhjgnujidhj
    jim@gmail.com:abc123
    sally@yahoo.com:cba321
    jim@gmail.com:abc123
    sally@yahoo.com:cba321
    jim@gmail.com:abc123
    sally@yahoo.com:cba321
    jim@gmail.com:abc123
    sally@yahoo.com:cba321
    jim@gmail.com:abc123
    sally@yahoo.com:cba321
    garbage text sdgusdfjhuhjgnujidhj
    """
    results = run_email(None, email_dummy, {})
    assert results["total_emails"] == 10
    assert results['unique_emails'] == 2
    assert results['unique_domains'] == 2


from elasticsearch import Elasticsearch
from pastehunter.common import parse_config
from datetime import datetime
import logging

logger = logging.getLogger('pastehunter')
config = parse_config()


class ElasticOutput():
    def __init__(self):
        # Set up the database connection
        es_host = config['outputs']['elastic_output']['elastic_host']
        es_port = config['outputs']['elastic_output']['elastic_port']
        es_user = config['outputs']['elastic_output']['elastic_user']
        es_pass = config['outputs']['elastic_output']['elastic_pass']
        self.es_index = config['outputs']['elastic_output']['elastic_index']
        self.weekly = config['outputs']['elastic_output']['weekly_index']
        es_ssl = config['outputs']['elastic_output']['elastic_ssl']
        self.test = False
        try:
            self.es = Elasticsearch(es_host, port=es_port, http_auth=(es_user, es_pass), use_ssl=es_ssl)
            self.test = True
        except Exception as e:
            logger.error(e)
            raise Exception('Unable to Connect') from None

    def store_paste(self, paste_data):
        if self.test:
            index_name = self.es_index
            if self.weekly:
                year_number = datetime.date(datetime.now()).isocalendar()[0]
                week_number = datetime.date(datetime.now()).isocalendar()[1]
                index_name = '{0}-{1}-{2}'.format(index_name, year_number, week_number)
            # ToDo: With multiple paste sites a pasteid collision is more likly!
            try:
                pasteid = str(paste_data['pasteid'])
                self.es.index(index=index_name, doc_type='paste', id=pasteid, body=paste_data)
                logger.debug("Stored {0} Paste {1}, Matched Rule {2}".format(paste_data['pastesite'],
                                                                             paste_data['pasteid'],
                                                                             paste_data['YaraRule']
                                                                             )
                             )
            except Exception as e:
                logger.error(e)
        else:
            logger.error("Elastic Search Enabled, not configured!")



import lzma
import base64
import logging
from pastehunter.common import parse_config
logger = logging.getLogger('pastehunter')
config = parse_config()

def run(results, raw_paste_data, paste_object):
    if config['outputs']['json_output']['store_raw']:
        original = raw_paste_data
        orig_size = len(original.encode())
        logger.debug("Compressing paste... Pre-compression size: {}", orig_size)
        compressed = base64.b64encode(lzma.compress(raw_paste_data.encode()))
        compressed_size = len(compressed)
        logger.debug("Compressing paste... Post-compression size: {}", compressed_size)

        # In some cases compressed blobs may be larger
        # if not much data is compressed
        if orig_size > compressed_size:
            paste_object['raw_paste'] = compressed.decode('utf-8')
            logger.debug("Compressed data smaller than original blob. Keeping compressed.")
        else:
            logger.debug("Original smaller than compressed blob. Keeping original.")

    # Regardless of modification, return the paste object
    return paste_object



import socket
from pastehunter.common import parse_config

config = parse_config()

class SyslogOutput():

    def store_paste(self, paste_data):
        host = config['outputs']['syslog_output']['host']
        port = config['outputs']['syslog_output']['port']

        syslog_line = '"{0}" "{1}" "{2}" "{3}" "{4}"'.format(paste_data['@timestamp'],
                                                paste_data['pasteid'],
                                                paste_data['YaraRule'],
                                                paste_data['scrape_url'],
                                                paste_data['pastesite'])
        syslog = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        syslog.connect((host, port))
        syslog.send(syslog_line.encode('utf-8'))
        syslog.close()



import logging
import math
import re
from datetime import datetime

import fnmatch
import requests

# Future work/improvement that can happen here: support PR diffs, they contain a patch URL
# Set some logging options
logger = logging.getLogger('pastehunter')
logging.getLogger('requests').setLevel(logging.ERROR)

api_uri = 'https://api.github.com/events'
# This event refers to a commit being pushed, and is
# probably the most significant thing we're concerned about.
event_types = ['PushEvent']
api_version = 'application/vnd.github.v3+json'  # Set Accept header to force api v3
# Important note from github:
# 'We delay the public events feed by five minutes, which means the most recent event returned by the public events API actually occurred at least five minutes ago.'

# Beware, git diffs can sometimes be very large files, including binaries and zips.
#                MB    KB     B
diff_size_limit = 500 * 1000 * 1000

blob_hash_pattern = re.compile('https://github.com/.*/blob/(.*?)/.*')


def _make_request(url, headers):
    req = requests.get(url, headers=headers)
    reset_date = datetime.utcfromtimestamp(float(req.headers['X-RateLimit-Reset'])).isoformat()
    logger.info('Remaining Limit: {0}. Resets at {1}'.format(req.headers['X-RateLimit-Remaining'],
                                                              reset_date))

    if req.status_code == 200:
        return req.json()

    if req.status_code == 401:
        logger.error('Auth Failed')
        return None

    elif req.status_code == 403:
        logger.error('Login Attempts Exceeded')
        return None

def get_blob_hash(file_dict):
    blob_url = file_dict.get('blob_url')
    return blob_hash_pattern.findall(blob_url)[0]

def recent_pastes(conf, input_history):
    oauth_token = conf['inputs']['github']['api_token']
    conf_limit = conf['inputs']['github']['api_limit']
    gh_limit = min(conf_limit, 300)
    # From GitHub Docs (https://developer.github.com/v3/activity/events/#list-public-events):
    # Events support pagination, however the per_page option is unsupported. The fixed page size is 30 items. Fetching up to ten pages is supported, for a total of 300 events.
    # We modify this to be 100 per page, but the limit is still 300.
    if gh_limit != conf_limit:
        logger.warning('gh_limit exceeds github items returned from public feed. Limiting to 300.')
    headers = {'user-agent': 'PasteHunter',
               'Accept': api_version,
               'Authorization': 'token {0}'.format(oauth_token)}

    # calculate number of pages
    page_count = int(math.ceil(gh_limit / 100))

    result_pages = []
    history = []
    paste_list = []

    gh_file_blacklist = conf['inputs']['github']['file_blacklist']
    gh_user_blacklist = conf['inputs']['github']['user_blacklist']
    ignore_bots = conf['inputs']['github']['ignore_bots']

    try:
        # Get the required amount of entries via pagination
        for page_num in range(1, page_count + 1):
            url = '{0}?page={1}&per_page=100'.format(api_uri, page_num)
            logger.debug('Fetching page: {0}'.format(page_num))
            req = _make_request(url, headers)
            if req is not None:
                result_pages.append(req)

        # Parse results

        for page in result_pages:
            for event_meta in page:
                # Track paste ids to prevent dupes
                event_id = event_meta['id']
                history.append(event_id)
                if event_id in input_history:
                    continue
                if event_meta['type'] not in event_types:
                    logger.debug('Skipping event {} due to unwanted type "{}"'.format(event_id, event_meta['type']))
                # Actor may have been deleted or changed
                if 'actor' in event_meta:
                    # If the username is None, this will return false, while event_meta['login'] would error.
                    if event_meta.get('actor').get('login') in gh_user_blacklist:
                        logger.info('Blacklisting GitHub event from user: {0}'.format(event_meta.get('login')))
                        continue
                    login = event_meta.get('actor').get('login')
                    if ignore_bots and login and login.endswith("[bot]"):
                        logger.info('Ignoring GitHub event from bot user: {}'.format(login))
                        continue

                payload = event_meta.get('payload')
                if not 'commits' in payload:
                    # Debug, because this is high output
                    logger.debug('Skipping event {} due to no commits.'.format(event_id))
                    continue
                for commit_meta in payload.get('commits'):
                    commit_url = commit_meta.get('url')
                    commit_data = _make_request(commit_url, headers)
                    if not commit_data:
                        logger.info('No data returned for url {}. Skipping...'.format(commit_url))
                        continue
                    if commit_data.get('committer') and commit_data.get('committer').get('login') in gh_user_blacklist:
                        logger.info('Blacklisting GitHub event from user: {0}'.format(event_meta['owner']['login']))
                        continue
                    for file_obj in commit_data.get('files'):
                        is_blacklisted = False
                        file_path = file_obj.get('filename')
                        for pattern in gh_file_blacklist:
                            if fnmatch.fnmatch(file_path, pattern):
                                logger.info('Blacklisting file {0} from event {1} (matched pattern "{2}")'.format(file_path, event_id, pattern))
                                is_blacklisted = True
                                break

                        if is_blacklisted:
                            continue

                        github_data = file_obj
                        github_data['confname'] = 'github'
                        github_data['@timestamp'] = event_meta['created_at']
                        github_data['pasteid'] = get_blob_hash(file_obj) or event_id
                        github_data['user'] = event_meta.get('actor').get('login')
                        github_data['pastesite'] = 'github.com'
                        github_data['scrape_url'] = file_obj.get('raw_url')
                        # remove some original keys just to keep it a bit cleaner
                        del github_data['raw_url']
                        paste_list.append(github_data)

        # Return results and history
        return paste_list, history
    except Exception as e:
        logger.exception('Unable to parse paste results: {0}'.format(e), e)
        return paste_list, history



#!/usr/bin/env python
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pastehunter',
    version='1.4.2',
    author='@kevthehermit @Plazmaz',
    author_email='info@pastehunter.com',
    description="Pastehunter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://pastehunter.com',
    license='GNU V3',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'yara-python',
        'requests',
        'elasticsearch',
        'splunk-sdk'
    ],
    scripts=['pastehunter-cli'],
    package_data={'': ['*.yar', 'README.md, LICENSE']}
)


import json
import logging
import os.path

logger = logging.getLogger('pastehunter')
home = os.path.expanduser("~")

BASE62_CHARS = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
BASE_LOOKUP = dict((c, i) for i, c in enumerate(BASE62_CHARS))
BASE_LEN = len(BASE62_CHARS)

# Parse the config file in to a dict
def parse_config():
    conf = None
    settings_file = os.path.join(home, ".config", "pastehunter.json")

    if os.path.exists(settings_file):
        conf_file = settings_file
    else:
        #ToDo: Copy base settings to the settings file
        conf_file = None

    if conf_file:
        try:
            with open(conf_file, 'r') as read_conf:
                conf = json.load(read_conf)
        except Exception as e:
            logger.error("Unable to parse config file: {0}".format(e))
    else:
        logger.error("Unable to read config file '~/.config/pastehunter.json'")

    return conf


# Most of this was pulled from https://stackoverflow.com/a/2549514
def base62_decode(input: str) -> int:
    length = len(BASE_LOOKUP)
    ret = 0
    for i, c in enumerate(input[::-1]):
        ret += (length ** i) * BASE_LOOKUP[c]

    return ret


def base62_encode(integer) -> str:
    if integer == 0:
        return BASE62_CHARS[0]

    ret = ''
    while integer != 0:
        ret = BASE62_CHARS[integer % BASE_LEN] + ret
        integer //= BASE_LEN

    return ret



from pastehunter.common import parse_config
import json
import logging
import splunklib.client as client

logger = logging.getLogger('pastehunter')
config = parse_config()

class SplunkOutput():
    def __init__(self):
        # Set up the database connection
        splunk_host = config['outputs']['splunk_output']['splunk_host']
        splunk_port = config['outputs']['splunk_output']['splunk_port']
        splunk_user = config['outputs']['splunk_output']['splunk_user']
        splunk_pass = config['outputs']['splunk_output']['splunk_pass']
        self.splunk_index = config['outputs']['splunk_output']['splunk_index']

        try:
            self.service = client.connect(
                host=splunk_host,
                port=splunk_port,
                username=splunk_user,
                password=splunk_pass,
                autologin=True)

            self.index = self.service.indexes[self.splunk_index]
        except Exception as e:
            logger.error(e)
            raise Exception('Unable to connect or missing index') from None

    def store_paste(self, paste_data):
        # Make a copy so we don't affect any other output modules
        local_data = dict(paste_data)
        if not config['outputs']['splunk_output']['store_raw']:
            del local_data['raw_paste']

        try:
            # The edit_tcp capability is required to access this API
            sourcetype = config['outputs']['splunk_output']['splunk_sourcetype']
            self.index.submit(json.dumps(local_data), sourcetype=sourcetype)
        except Exception as e:
            logger.exception('Error submitting paste_data to splunk', e)



import logging

import requests

from pastehunter.common import parse_config

logger = logging.getLogger('pastehunter')

config = parse_config()

class HttpOutput():
    def __init__(self):
        self.valid = True
        self.endpoint_url = config['outputs']['http_output']['endpoint_url']
        self.http_headers = config['outputs']['http_output']['headers']
        self.http_auth = config['outputs']['http_output']['http_auth']
        self.http_user = config['outputs']['http_output']['http_user']
        self.http_password = config['outputs']['http_output']['http_password']
        self.ignore_fields = config['outputs']['http_output']['ignore_fields']
        self.timezone = config['outputs']['http_output']['timezone']

        if self.endpoint_url == '':
            logging.error("HTTP endpoint not configured")
            self.valid = False

    def store_paste(self, paste_data):
        if self.valid:

            json_data = paste_data

            for field in self.ignore_fields:
                del json_data[field]

            json_data['@timestamp'] += self.timezone

            if self.http_auth:
                req = requests.post(self.endpoint_url, headers=self.http_headers, json=json_data, auth=(self.http_user, self.http_password))
            else:
                req = requests.post(self.endpoint_url, headers=self.http_headers, data=json_data)

            if req.status_code == 200 or req.status_code == 201:
                logger.debug("Paste sent to HTTP endpoint")
            else:
                logger.error("Failed to post to HTTP endpoint {0}".format(req.status_code))



import re
import math
from collections import Counter

def shannon_entropy(s):
    # https://rosettacode.org/wiki/Entropy#Python
    s = str(s)
    p, lns = Counter(s), float(len(s))
    return -sum(count / lns * math.log(count / lns, 2) for count in p.values())


def run(results, raw_paste_data, paste_object):
    # Calculate the Shannon Entropy for the raw paste
    paste_object["Shannon Entropy"] = shannon_entropy(raw_paste_data)
    # Send the updated json back
    return paste_object



from abc import ABC, abstractmethod
from typing import Any, Optional, Dict, List, Union

import requests


class BasePasteSite(ABC):
    def make_request(self, url: str, timeout: Optional[int] = 10, headers: Optional[Dict[str, Any]] = None):
        """
        Make a request and return the results
        :param url: The url to request
        :param timeout: The timeout for the request
        :param headers: The headers dict
        :return:
        """
        req = requests.get(url, headers=headers, timeout=timeout)
        return req

    @abstractmethod
    def remap_raw_item(self, raw_item: [str, Dict]) -> Dict[str, Any]:
        """
        Takes a raw item and remaps it to a normalize paste dict
        :param raw_item:
        :return: The paste dict
        """
        pass

    @abstractmethod
    def get_paste_for_id(self, paste_id: Any) -> str:
        """
        Returns a paste for the given paste_id
        :param paste_id: The paste to retrieve
        :return: A raw paste object
        """
        pass

    @abstractmethod
    def get_paste_id(self, paste_obj: Dict[str, Any]) -> Union[str, int]:
        """
        Returns an id for the given paste object
        :param paste_obj: The raw paste dict
        :return: The paste id
        passd (str or int)
        """

    @abstractmethod
    def get_recent_items(self, input_history: List[str]):
        """
        Gets recent items
        :return: a list of recent items
        """
        pass



import io
import logging
import requests
from pastehunter.common import parse_config
conf = parse_config()

logger = logging.getLogger('pastehunter')

def upload_file(raw_file, paste_object):
    try:
        task_id = send_to_cuckoo(raw_file, paste_object["pasteid"])
        paste_object["Cuckoo Task ID"] = task_id
        logger.info("exe submitted to Cuckoo with task id {0}".format(task_id))
    except Exception as e:
        logger.error("Unabled to submit sample to cuckoo")

    # Send any updated json back
    return paste_object

def send_to_cuckoo(raw_exe, pasteid):
    cuckoo_ip = conf["sandboxes"]["cuckoo"]["api_host"]
    cuckoo_port = conf["sandboxes"]["cuckoo"]["api_port"]
    cuckoo_host = 'http://{0}:{1}'.format(cuckoo_ip, cuckoo_port)
    submit_file_url = '{0}/tasks/create/file'.format(cuckoo_host)
    files = {'file': ('{0}.exe'.format(pasteid), io.BytesIO(raw_exe))}
    submit_file = requests.post(submit_file_url, files=files).json()
    task_id = None
    try:
        task_id = submit_file['task_id']
    except KeyError:
        try:
            task_id = submit_file['task_ids'][0]
        except KeyError:
            logger.error(submit_file)

    return task_id



from inputs.pastebin import PastebinPasteSite
from inputs.slexy import SlexyPasteSite

pids_found = []


def mock_get_paste_for_pid(pid):
    pids_found.append(pid)
    return "pid_is_" + pid


class FakeRequestJson(object):
    def __init__(self, ret):
        self.ret = ret

    def json(self):
        return self.ret


def test_slexy_site():
    pid_list_fake = [0, 1, 2, 3, 4]
    slexy_site = SlexyPasteSite({})
    slexy_site.get_recent_items = lambda: pid_list_fake
    slexy_site.get_paste_for_id = lambda pid: mock_get_paste_for_pid(str(pid))
    slexy_site.remap_raw_item = lambda raw_data, pid: {"pid": 123}
    recent_pids = slexy_site.get_recent_items()
    assert recent_pids == pid_list_fake
    for pid in recent_pids:
        paste = slexy_site.get_paste_for_id(pid)
        paste_data = slexy_site.remap_raw_item(paste, pid)
        assert paste == 'pid_is_' + str(pid)
        assert paste_data == {"pid": 123}


def test_pastebin_site_remap():
    fake_conf = {
        'inputs': {
            'pastebin': {
                'paste_limit': 100,
                'api_scrape': 'https://scrape.pastebin.com/api_scraping.php'
            }
        }
    }
    data = {
        'key': 'a',
        'test': 'b',
        'date': '1582595793'
    }
    pastebin_site = PastebinPasteSite(fake_conf)
    out = pastebin_site.remap_raw_item(data)
    assert out == {'key': 'a', 'test': 'b', 'date': '1582595793', 'filename': 'a', 'confname': 'pastebin',
                   'pasteid': 'a', 'pastesite': 'pastebin.com', '@timestamp': '2020-02-25T01:56:33'}


def test_pastebin_site():
    fake_conf = {
        'inputs': {
            'pastebin': {
                'paste_limit': 100,
                'api_scrape': 'https://scrape.pastebin.com/api_scraping.php'
            }
        }
    }
    pastebin_site = PastebinPasteSite(fake_conf)
    pastebin_site.make_request = lambda url: FakeRequestJson([
        {
            'key': 'ab',
            'date': '1582595793'
        },
        {
            'key': 'bc',
            'date': '1582595793'
        }
    ])
    pastes, paste_ids = pastebin_site.get_recent_items([])
    assert paste_ids == ['ab', 'bc']
    assert pastes[0].get('key') == 'ab'
    assert pastes[1].get('key') == 'bc'



import hashlib
import importlib
import gzip
import logging
from base64 import b64decode
# This gets the raw paste and the paste_data json object
from pastehunter.common import parse_config
conf = parse_config()

logger = logging.getLogger('pastehunter')

def run(results, raw_paste_data, paste_object):

    '''

    ToDo: Lets look at multiple base64 streams
    for now only accept if the entire paste is

    # Figure out which b64 rule fire

    # The base64 re can hang on occasion with this one
    # b64_re = '(?:[A-Za-z0-9+/]{4}){2,}(?:[A-Za-z0-9+/]{2}[AEIMQUYcgkosw048]=|[A-Za-z0-9+/][AQgw]==)'

    # This one has a few empty results i need to catch but doesn't kill pastehunter
    b64_re = '(?:[A-Za-z0-9+/]{4}){3,}(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?'
    b64_strings = re.findall(b64_re, raw_paste_data)


    # Set a counter for multiple streams.
    counter = 0
    for b64_str in b64_strings:

    '''

    for rule in results:
        if len(raw_paste_data) > 0:
            if rule == 'b64_gzip':
                # Lets try to decode and get a file listing.
                # Also get the MD5 of the decoded file
                try:
                    uncompressed = gzip.decompress(b64decode(raw_paste_data))
                    encoded = uncompressed.encode('utf-8')
                    paste_object["decompressed_stream"] = encoded
                except Exception as e:
                    logger.error("Unable to decompress gzip stream")

            if rule == 'b64_exe':
                try:
                    raw_exe = b64decode(raw_paste_data)
                    paste_object["exe_size"] = len(raw_exe)
                    paste_object["exe_md5"] = hashlib.md5(raw_exe).hexdigest()
                    paste_object["exe_sha256"] = hashlib.sha256(raw_exe).hexdigest()

                    # We are guessing that the sample has been submitted, and crafting a URL
                    paste_object["VT"] = 'https://www.virustotal.com/#/file/{0}'.format(paste_object["exe_md5"])

                    # If sandbox modules are enabled then submit the file
                    for sandbox, sandbox_values in conf["sandboxes"].items():
                        if sandbox_values["enabled"]:
                            logger.info("Uploading file {0} using {1}".format(paste_object["pasteid"], sandbox_values["module"]))
                            sandbox_module = importlib.import_module(sandbox_values["module"])
                            paste_object = sandbox_module.upload_file(raw_exe, paste_object)

                except Exception as e:
                    logger.error("Unable to decode exe file")

    # Get unique domain count
    # Update the json

    # Send the updated json back
    return paste_object
